import React, { useState, useEffect } from 'react';
import { getAvailability, setAvailability } from './api';

const NameForm = ({ onNameSet }) => {
  const [name, setName] = useState("");

  return (
    <div>
      <form onSubmit={() => onNameSet(name.trim())}>
        <input 
          type="text" 
          onChange={({ target }) => setName(target.value)}
          value={name}
        />
        <input 
          disabled={name === ""} 
          type="submit" 
          value="Submit"
        />
      </form>
    </div>
  )
}

const colors = {
  yes: "#0f0",
  no: "#f00",
  maybe: "#ff0",
}

const nextState = {
  yes: "no",
  no: "maybe",
  maybe: "unknown",
  unknown: "yes",
}

const pad = num => {
  let snum = "" + num
  if (snum.length === 1) snum = "0" + snum
  return snum
}

const Tile = ({ user, month, day, availability, onClick }) => (
  <div 
    onClick={onClick}
    style={{
      display: "flex",
      flexFlow: "column nowrap",
      width: "5em",
      height: "5em",
      border: "1px solid black",
      margin: "2px 2px 2px 2px",
      backgroundColor: colors[availability.filter(({ name }) => name === user)[0]?.status] || "#fff",
    }}
  >
    <span>{pad(month)}/{pad(day)}</span>
    {
      availability
        .filter(({ status }) => status && status !== "unknown")
        .filter(({ name }) => name !== user)
        .map(({ name, status }) => (
          <span 
            key={name} 
            style={{ color: colors[status] }}
          >
            {name}
          </span>
        ))
      }
  </div>
);


const chunk = (arr, len) => {
  const chunks = [];
  for (let i = 0; i < arr.length;) {
    chunks.push(arr.slice(i, i += len));
  }
  return chunks;
}

function App() {
  const [name, setName] = useState(null);
  const [calendar, setCalendar] = useState(null);
  useEffect(() => { getAvailability().then(({availability}) => setCalendar(availability)) }, []);
  const advance = async (dates) => {
    const updates = dates.map(({ month, day }) => {
      const currentStatus = calendar
        .find(item => item.month === month && item.day === day)
        ?.availability
        ?.find(item => item.name === name)
        ?.status 
        ?? "unknown"
      const status = nextState[currentStatus] || "unknown";
      return { month, day, status }
    });
    const { availability } = await setAvailability(name, updates);
    setCalendar(availability);
  }


  if (name === null) {
    return <NameForm onNameSet={setName} />
  }

  if (calendar === null) {
    return <span>Loading...</span>
  }

  return (
    <div
      style={{
        display: "flex",
        flexFlow: "column nowrap",
      }}
    >
      <span>Logged in as: {name}</span>
      {
        chunk(calendar, 7).map((row, ind) => (
          <div
            key={ind} 
            style={{
              display: "flex",
              flexFlow: "row nowrap",
              alignItems: "center",
            }}
          >
            <div
              onClick={() => advance(row)}
            >
              WEEK
            </div>
            <div 
              style={{
                display: "flex",
                flexFlow: "row nowrap",
              }}
            >
              {
                row.map(item => <Tile onClick={() => advance([item])} key={item.day} user={name} {...item}/>)
              }
            </div>
          </div>
        ))
      }
    </div>
  );
}

export default App;
